//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013-2015 Nicolas Froment, Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================
//
// History:
//
// 2.1  Jul 2015 by Djamana
//  * Rainbow scale applied to circle of fifth┤s
//
// 2.0  Jul 2015 by Djamana
//  * cleaned up & reordered source code
//  * #,b and . color is now always linked to the note
//  * color also the stem and beam of the notes
// 
import QtQuick 2.0
import MuseScore 1.0

MuseScore {
      version:  "2.0"
      description: "This demo plugin colors notes in the selection depending on pitch"
      menuPath: "Plugins.Notes.Color Notes v2"


// Rainbow scale applied to circle of fifth┤s (started on C with magenta)
// 
// http://virtuosoism.com/Color-Sound%20Relationships.pdf 
// (Combined with with http://www.color-picker.de)
  
      property variant colors : [
	/*C*/   qColor(226, 48,140), qColor(  0,175,136),
	/*D*/   qColor(243,111, 45), qColor(  0,133,202),
       /*E*/   qColor(247,232, 41), 
	/*F*/   qColor(139, 75,157), qColor(105,190, 87),
	/*G*/   qColor(240, 65, 72), qColor(  0,176,200),
	/*A*/   qColor(251,169, 37), qColor( 68, 98,174),
	/*B*/   qColor(198,217, 55)

/*
// old/default Version colormap - Rainbow colors on cromatic scale (start on c with red)
					"#e21c48", "#f26622",    //C  
					"#f99d1c", "#ffcc33",    //D  
					"#fff32b",               //E
					"#bcd85f", "#62bc47",    //F 
					"#009c95", "#0071bb",    //G
					"#5e50a1", "#8d5ba6",    //A
					"#cf3e96"                //B
*/

      ]
      property variant black : "#000000"


      function colorNote(note) {
		  

		// Color black notes and
		// make colored notes black
		if (note.color == black)
			
			// Select color according to pitch
			noteColor = colors[note.pitch % 12]
		else
			noteColor = black;

	 // color note		
		applyColorOn( note );

	 // color # or b
		applyColorOn( note.accidental );

	 // color . or ..
		forEach ( note.dots, applyColorOn );


/////////////////////////////////////////////////////

        //
        // color also the stem and beam of the note
        //
            var the = cursor.element;
            applyColorOn( the.stem );
            applyColorOn( the.hook);      
            applyColorOn( the.beam);      
            applyColorOn( the.stemSlash);      
         // ^- UNCOMMENT/delete these lines if you don't like this!

///////////////////////////////////////////////////////



	 //////////////////////////////////////
	 // Helper function [closure]
		var noteColor;

		function applyColorOn( myObj ) {
			if (myObj) {
				myObj.color = noteColor;
			}
		}
		  
	 }



      // Apply the given function to all notes in selection
      // or, if nothing is selected, in the entire score

      property variant cursor : curScore.newCursor();
 

      function applyToNotesInSelection() {

      var rewind = {    score_start       : 0,
                        selection_start   : 1,
                        selection_end     : 2
                   }


            var startStaff = 0; // start with 1st staf
            var endStaff = curScore.nstaves - 1; // and end with last

            var endTick = 0xffffffff;

           cursor.rewind(rewind.selection_start);

           var fullScore = !cursor.segment;


            if (!fullScore) {
                  startStaff = cursor.staffIdx;

                  cursor.rewind( rewind.selection_end );
                  if (cursor.tick == 0) {
                        // this happens when the selection includes
                        // the last measure of the score.
                        // rewind( rewind.selection_end ) goes behind the last segment (where
                        // there's none) and sets tick=0
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }

                  endStaff = cursor.staffIdx;
            }

            console.log(startStaff + " - " + endStaff + " : ..." + endTick)

            // Loop through staffs, voices, measures...
            for (var staff = startStaff; staff <= endStaff; staff++) {


                  for (var voice in [1,2,3,4]) {

	                  cursor.rewind(rewind.selection_start);

                        if (fullScore)
                              cursor.rewind(rewind.score_start) // if no selection, beginning of score

                     // Attention: voice & staff has to be set after 'goTo '
                        cursor.staffIdx = staff;
                        cursor.voice = voice;

                     // Loop through measures ...
                        while ( cursor.segment && 
                               (cursor.tick < endTick) ) {

                              
                              if (cursor.element && 
                                  cursor.element.type == Element.CHORD) {
									  

                                    forEach(cursor.element.graceNotes, 
                                          function (graceChords){
                                                // iterate through all grace chords 
                                                forEach(graceChords.notes, colorNote)
                                          })


                                    forEach(cursor.element.notes, colorNote)

                              }

                              cursor.next();
                        }
                  }
            }
      }


	// Some Helper function
	function forEach(items, func) {
		  for (var i in items)
				func(items[i]);
	}

      function qColor(r,g,b) {
            var rgb2hex; 
                rgb2hex = "#" + Number(
                  0x1000000 + 
                  r*0x10000 + 
                  g*0x100 + 
                  b).toString(16)
            .substring(1);

    return (rgb2hex);

}

		 
      onRun: {
            console.log("hello colornotes");

            if (typeof curScore !== 'undefined')
                 applyToNotesInSelection()

            Qt.quit();
         }
}
